#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base Redborder Module
#
module Redborder
  #
  # Base Location Module
  #
  module Location
    #
    # Location QueryManager implement methods to perform module queries
    #
    module QueryManager
      NUM_SECONDS = 600_00

      private

      def movements_data
        filter_tab
        parser = Redborder::Location::MovementsParser.new(movements_query, @locations)
        { chord: parser.to_chord, table: parser.to_movements_table }
      end

      def total_data
        total_query
      end

      def tops_data
        @filter = @filter.dup
        adjust_client_profile_filter(@client_profile) if params[:w]
        @filter.add(:transition, @source::TRANSITION[:STATIC])
        filter_tab
        data = params[:w] == 'true' ? tops_query_to_widget : tops_query
        if @aggregation == :cumulative_dwell_time
          minutes = Redborder::Rbdruid::GRANULARITY_TYPE[@granularity][:step] / NUM_SECONDS
          data.each { |d| d.cumulative_dwell_time /= minutes }
        elsif @aggregation == :clients
          data.each { |d| d.clients = d.clients.round }
        end
        data
      end

      def serie_data(comparative=false)
        @filter = @filter.dup
        adjust_client_profile_filter(@client_profile) if params[:w]
        if @tab == :dot11_status and comparative
          @tab = :status
        end
        if params[:w] != "true"
          @source::TABS[@tab][:columns].each do |tab|
            @filter.remove(tab)
          end
        end
        filter_tab
        @filter.add(:transition, @source::TRANSITION[:STATIC])

        if !comparative && (params[:w].nil? || params[:w] != "true")
          @source::TABS[@tab][:columns].each { |column| @filter.add(column, @uuid) }
        end

        # idea, refactor this
        if comparative
          @granularity = case params[:id].to_sym
                 when :pt24h then :pt1m
                 when :pt168h then :pt5m
                 when :pt720h then :hour
                 when :pt4320h then :pt8h
                 end
        end

        data = serie_query
        # idea, refactor this
        if @granularity != :minute && @aggregation == :cumulative_dwell_time
          minutes = Redborder::Rbdruid::GRANULARITY_TYPE[@granularity][:step] / NUM_SECONDS
          data.each { |d| d[:event][:events] /= minutes }
        end
        data
      end

      def map_data
        @filter.add(:transition, @source::TRANSITION[:STATIC])
        @filter.exclude(:client_latlong, nil)
        map_query
      end

      ##
      ## Queries Methods
      ##

      def total_query
        @source.total(
          start_time: @start_time,
          end_time: @end_time,
          aggregation: @aggregation,
          search: @filter.get
        )
      end

      def serie_query
        @source.timeserie(
          start_time: @start_time,
          end_time: @end_time,
          range: @range || :custom,
          granularity: @granularity,
          search: @filter.get,
          aggregation: @aggregation
        )
      end

      def map_query
        @source.where(
          query_type: :group_by,
          range: @range,
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all,
          group_by: [:client_latlong],
          search: @filter.get,
          aggregation: :clients
        )
      end

      def movements_query
        @source.where(
          query_type: :group_by,
          range: @range,
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all,
          group_by: [:old, :new],
          search: @filter.get,
          aggregation: :clients
        )
      end

      def tops_query
        @source.topN(
          query_type: :group_by,
          start_time: @start_time,
          end_time: @end_time,
          range: @range || :custom,
          granularity: :all,
          limit: (@load_more_offset || 1) * current_user.per_page_count,
          group_by: @source::TABS[@tab][:columns],
          search: @filter.get,
          aggregation: @aggregations
        )
      end

      def tops_query_to_widget
        @source.topN(
          query_type: :group_by,
          start_time: @start_time,
          end_time: @end_time,
          range: @range || :custom,
          granularity: :all,
          limit: (@load_more_offset || 1) * current_user.per_page_count,
          group_by: @source::TABS[@tab][:columns],
          search: @filter.get,
          aggregation: @aggregation
        )
      end

      def filter_tab
        tab_string = @tab.to_s
        if %w[status client_mac client_mac_vendor calendar_item_name].include?(tab_string)
          # filter by Floor at Status Tab as it's the most restrictive
          @filter.add(:type, 'floor')
        else
          @filter.add(:type, tab_string)
        end
      end

      private

      def adjust_client_profile_filter(client_profile)
        case client_profile
        when 'hard'
          @filter.add(:client_profile, 'hard')
        when 'medium'
          @filter.add(:client_profile, ['medium', 'hard'])
        when 'soft'
          # No hacer nada si es 'soft'
        end
      end
    end
  end
end
